# Generated by Django 3.0 on 2021-04-15 16:29

from django.db import migrations
from django.db import models


class Migration(migrations.Migration):

    dependencies = [
        ('tesla_ce', '0011_auto_20200930_1842'),
    ]

    operations = [
        migrations.AddField(
            model_name='institutionuser',
            name='data_admin',
            field=models.BooleanField(default=False, help_text='Whether this user can manage the data of the institution'),
        ),
        migrations.AlterField(
            model_name='enrolmentsamplevalidation',
            name='status',
            field=models.IntegerField(choices=[(0, 'Validating'), (1, 'Valid'), (2, 'Error'), (3, 'Timeout'), (4, 'Waiting External Service')], default=0, help_text='Validation status for this sample'),
        ),
        migrations.AlterField(
            model_name='requestproviderresult',
            name='status',
            field=models.IntegerField(choices=[(0, 'Pending'), (1, 'Processed'), (2, 'Error'), (3, 'Timeout'), (4, 'Missing Provider'), (5, 'Missing Enrolment'), (6, 'Processing'), (7, 'Waiting External Service')], default=0, help_text='Status for this result'),
        ),
        migrations.AlterField(
            model_name='requestresult',
            name='status',
            field=models.IntegerField(choices=[(0, 'Pending'), (1, 'Processed'), (2, 'Error'), (3, 'Timeout'), (4, 'Missing Provider'), (5, 'Missing Enrolment'), (6, 'Processing'), (7, 'Waiting External Service')], default=0, help_text='Status for this result'),
        ),
    ]
